@@ -34,7 +34,6 @@ NULL
 #' This functions is a constructor for the \code{cor_arma} class, representing 
 #' an autoregression-moving average correlation structure of order (p, q).
 #' 
-#' @aliases cor.arma
 #' @aliases cor_arma-class
 #' 
 #' @param formula A one sided formula of the form \code{~ t}, or \code{~ t | g}, 
@@ -63,7 +62,7 @@ NULL
 #'   autoregression-moving-average correlation structure.
 #' 
 #' @details AR refers to autoregressive effects of residuals, which
-#'   is what is typcially understood as autoregressive effects.
+#'   is what is typically understood as autoregressive effects.
 #'   However, one may also model autoregressive effects of the response
 #'   variable, which is called ARR in \pkg{brms}.
 #' 
@@ -96,25 +95,18 @@ cor_arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
   x
 }
 
-#' @export
-cor.arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
-  warn_deprecated("cor_arma")
-  cor_arma(formula = formula, p = p, q = q, r = r, cov = cov)
-}
-
 #' AR(p) correlation structure
 #' 
 #' This function is a constructor for the \code{cor_arma} class, 
 #' allowing for autoregression terms only.
-#' 
-#' @aliases cor.ar
+
 #' 
 #' @inheritParams cor_arma
 #' 
 #' @return An object of class \code{cor_arma} containing solely autoregression terms.
 #' 
 #' @details AR refers to autoregressive effects of residuals, which
-#'   is what is typcially understood as autoregressive effects.
+#'   is what is typically understood as autoregressive effects.
 #'   However, one may also model autoregressive effects of the response
 #'   variable, which is called ARR in \pkg{brms}.
 #' 
@@ -129,20 +121,12 @@ cor.arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
 cor_ar <- function(formula = ~ 1, p = 1, cov = FALSE) {
   cor_arma(formula = formula, p = p, q = 0, r = 0, cov = cov)
 }
-
-#' @export
-cor.ar <- function(formula = ~ 1, p = 1, cov = FALSE) {
-  warn_deprecated("cor_ar")
-  cor_ar(formula = formula, p = p, cov = cov)
-}
   
 #' MA(q) correlation structure
 #' 
 #' This function is a constructor for the \code{cor_arma} class, 
 #' allowing for moving average terms only.
 #' 
-#' @aliases cor.ma
-#' 
 #' @inheritParams cor_arma
 #' 
 #' @return An object of class \code{cor_arma} containing solely moving average terms.
@@ -159,12 +143,6 @@ cor_ma <- function(formula = ~ 1, q = 1, cov = FALSE) {
   cor_arma(formula = formula, p = 0, q = q, r = 0, cov = cov)
 }
 
-#' @export
-cor.ma <- function(formula = ~ 1, q = 1, cov = FALSE) {
-  warn_deprecated("cor_ma")
-  cor_ma(formula = formula, q = q, cov = cov)
-}
-
 #' ARR(r) correlation structure
 #' 
 #' This function is a constructor for the \code{cor_arma} class 
@@ -176,7 +154,7 @@ cor.ma <- function(formula = ~ 1, q = 1, cov = FALSE) {
 #'   autoregressive response terms.
 #'   
 #' @details AR refers to autoregressive effects of residuals, which
-#'   is what is typcially understood as autoregressive effects.
+#'   is what is typically understood as autoregressive effects.
 #'   However, one may also model autoregressive effects of the response
 #'   variable, which is called ARR in \pkg{brms}.
 #' 
@@ -201,7 +179,7 @@ cor_arr <- function(formula = ~ 1, r = 1) {
 #' The \code{errorsar} structure implements SAR of the residuals:
 #' \deqn{y = \eta + u, u = \rho W u + e}
 #' In the above equations, \eqn{\eta} is the predictor term and
-#' \eqn{e} are independent normally or t-distribued residuals.
+#' \eqn{e} are independent normally or t-distributed residuals.
 #' 
 #' @param W An object specifying the spatial weighting matrix.
 #'   Can be either the spatial weight matrix itself or an 
@@ -281,7 +259,7 @@ sar_weights <- function(W) {
 #' @param W Adjacency matrix of locations. 
 #'   All non-zero entries are treated as if the two locations 
 #'   are adjacent. If \code{formula} contains a grouping factor,
-#'   the row names of \code{W} have to matche the levels
+#'   the row names of \code{W} have to match the levels
 #'   of the grouping factor.
 #' @param formula An optional one-sided formula of the form 
 #'   \code{~ 1 | g}, where \code{g} is a grouping factor mapping
@@ -289,7 +267,7 @@ sar_weights <- function(W) {
 #'   each observation is treated as a separate location.
 #'   It is recommended to always specify a grouping factor
 #'   to allow for handling of new data in post-processing methods.
-#' @param type Type of the CAR structure. Currenlty implemented
+#' @param type Type of the CAR structure. Currently implemented
 #'   are \code{"escar"} (exact sparse CAR) and \code{"esicar"}
 #'   (exact sparse intrinsic CAR). More information is
 #'   provided in the 'Details' section.
@@ -398,12 +376,6 @@ cor_fixed <- function(V) {
   structure(list(V = V), class = c("cor_fixed", "cor_brms"))
 }
 
-#' @export
-cov_fixed <- function(V) {
-  warn_deprecated("cor_fixed")
-  cor_fixed(V)
-}
-
 #' Basic Bayesian Structural Time Series
 #' 
 #' Add a basic Bayesian structural time series component to a brms model
@@ -561,7 +533,7 @@ use_cov <- function(x) {
 }
 
 stop_not_cor_brms <- function(x) {
-  if (!(is.null(x) || is.cor_brms(x) || is(x, "cor.brms"))) {
+  if (!(is.null(x) || is.cor_brms(x))) {
     stop2("Argument 'autocor' must be of class 'cor_brms'.")
   }
   TRUE
@@ -581,13 +553,23 @@ check_autocor <- function(autocor) {
   autocor
 }
 
-remove_autocor <- function(x, keep = FALSE) {
+remove_autocor <- function(x) {
   # convenience function to ignore autocorrelation terms
   # currently excludes ARMA, SAR, and CAR structures
-  excl_cor <- is.cor_arma(x$autocor) || 
-    is.cor_sar(x$autocor) || is.cor_car(x$autocor)
-  if (!keep && excl_cor) {
-    x$autocor <- x$formula$autocor <- cor_empty()
+  if (is_mv(x)) {
+    for (r in names(x$formula$forms)) {
+      ac <- x$formula$forms[[r]]$autocor
+      excl_cor <- is.cor_arma(ac) || is.cor_sar(ac) || is.cor_car(ac)
+      if (excl_cor) {
+        x$autocor[[r]] <- x$formula$forms[[r]]$autocor <- cor_empty()
+      }
+    }
+  } else {
+    ac <- x$formula$autocor
+    excl_cor <- is.cor_arma(ac) || is.cor_sar(ac) || is.cor_car(ac)
+    if (excl_cor) {
+      x$autocor <- x$formula$autocor <- cor_empty()
+    }
   }
   x
 }
