@@ -50,7 +50,7 @@ brmsfit <- function(formula = NULL, family = NULL, data = data.frame(),
                     data.name = "", model = "", prior = empty_brmsprior(), 
                     autocor = NULL, ranef = empty_ranef(), 
                     cov_ranef = NULL, loo = NULL, waic = NULL, R2 = NULL,
-                    bridge = NULL, fit = NA, exclude = NULL, 
+                    bridge = NULL, stan_funs = NULL, fit = NA, exclude = NULL, 
                     algorithm = "sampling") {
   # brmsfit class
   version <- list(
@@ -60,7 +60,7 @@ brmsfit <- function(formula = NULL, family = NULL, data = data.frame(),
   x <- nlist(
     formula, family, data, data.name, model, prior, 
     autocor, ranef, cov_ranef, loo, waic, R2, bridge, 
-    fit, exclude, algorithm, version
+    stan_funs, fit, exclude, algorithm, version
   )
   class(x) <- "brmsfit"
   x
@@ -106,13 +106,13 @@ is.brmsfit <- function(x) {
 #'  the evidence ratio is the ratio of the posterior probability 
 #'  of \code{a > b} and the posterior probability of \code{a < b}.
 #'  In this example, values greater than one indicate that the evidence in
-#'  favour of \code{a > b} is larger than evidence in favour of \code{a < b}.
+#'  favor of \code{a > b} is larger than evidence in favor of \code{a < b}.
 #'  For an undirected (point) hypothesis, the evidence ratio 
 #'  is a Bayes factor between the hypothesis and its alternative
 #'  computed via the Savage-Dickey density ratio method.
 #'  That is the posterior density at the point of interest divided
 #'  by the prior density at that point.
-#'  Values greater than one indicate that evidence in favour of the point
+#'  Values greater than one indicate that evidence in favor of the point
 #'  hypothesis has increased after seeing the data.
 #'  In order to calculate this Bayes factor, all parameters related 
 #'  to the hypothesis must have proper priors
@@ -184,7 +184,7 @@ hypothesis <- function(x, ...) {
   UseMethod("hypothesis")
 }
 
-#' Decriptions of \code{brmshypothesis} Objects
+#' Descriptions of \code{brmshypothesis} Objects
 #' 
 #' A \code{brmshypothesis} object contains posterior samples
 #' as well as summary statistics of non-linear hypotheses as 
@@ -226,7 +226,6 @@ NULL
 #' @param pars Names of parameters for which posterior samples 
 #'   should be returned, as given by a character vector or regular expressions.
 #'   By default, all posterior samples of all parameters are extracted.
-#' @param parameters A deprecated alias of \code{pars}.  
 #' @param exact_match Indicates whether parameter names 
 #'   should be matched exactly or treated as regular expression. 
 #'   Default is \code{FALSE}.
@@ -234,7 +233,6 @@ NULL
 #'   should contain two additional columns. The \code{chain} column 
 #'   indicates the chain in which each sample was generated, the \code{iter} 
 #'   column indicates the iteration number within each chain.
-#' @param add_chains A deprecated alias of \code{add_chain}.
 #' @param subset A numeric vector indicating the rows 
 #'   (i.e., posterior samples) to be returned. 
 #'   If \code{NULL} (the default), all  posterior samples are returned.
@@ -277,16 +275,6 @@ posterior_samples <- function(x, pars = NA, ...) {
   UseMethod("posterior_samples")
 }
 
-#' @export 
-posterior.samples <- function(x, pars = NA, ...) {
-  # deprecated alias of posterior_samples
-  warning2(
-    "Method 'posterior.samples' is deprecated. ", 
-    "Please use method 'posterior_samples' instead." 
-  )
-  UseMethod("posterior_samples")
-}
-
 #' Extract prior samples
 #' 
 #' Extract prior samples of specified parameters 
@@ -297,7 +285,6 @@ posterior.samples <- function(x, pars = NA, ...) {
 #' @param pars Names of parameters for which prior samples should be returned, 
 #'   as given by a character vector or regular expressions.
 #'   By default, all prior samples are extracted
-#' @param parameters A deprecated alias of \code{pars}       
 #' @param ... Currently ignored
 #'   
 #' @details To make use of this function, 
@@ -650,7 +637,7 @@ reloo <- function(x, ...) {
 #'   How this variable is handled depends on argument \code{exact_loo}.
 #'   If \code{exact_loo} is \code{FALSE}, the data is split 
 #'   up into subsets, each time omitting all observations of one of the 
-#'   factor levels, while ignoriing argument \code{K}. 
+#'   factor levels, while ignoring argument \code{K}. 
 #'   If \code{exact_loo} is \code{TRUE}, all observations corresponding 
 #'   to the factor level of the currently predicted single value are omitted. 
 #'   Thus, in this case, the predicted values are only a subset of the 
@@ -691,16 +678,6 @@ kfold <- function(x, ...) {
   UseMethod("kfold")
 }
 
-#' @export
-launch_shiny <- function(object, rstudio = getOption("shinystan.rstudio"), ...) {
-  # deprecated alias of launch_shinystan
-  warning2(
-    "Method 'launch_shiny' is deprecated. ", 
-    "Please use method 'launch_shinystan' instead."
-  )
-  UseMethod("launch_shinystan")
-}
-
 #' Extract Stan Model Code
 #' 
 #' Extract the model code in Stan language
@@ -735,7 +712,7 @@ standata <- function(object, ...) {
 
 #' MCMC Plots Implemented in \pkg{bayesplot} 
 #' 
-#' Conveniant way to call MCMC plotting functions 
+#' Convenient way to call MCMC plotting functions 
 #' implemented in the \pkg{bayesplot} package. 
 #' 
 #' @inheritParams posterior_samples
@@ -763,7 +740,7 @@ standata <- function(object, ...) {
 #' 
 #' @details 
 #'   Also consider using the \pkg{shinystan} package available via 
-#'   method \code{\link[brms:launch_shiny]{launch_shiny}} 
+#'   method \code{\link{launch_shinystan}}. 
 #'   in \pkg{brms} for flexible and interactive visual analysis. 
 #' 
 #' @examples
@@ -807,7 +784,7 @@ stanplot <- function(object, ...) {
 #'   If \code{NULL} (the default), plots are generated for all main effects
 #'   and two-way interactions estimated in the model. When specifying
 #'   \code{effects} manually, \emph{all} two-way interactions may be plotted
-#'   even if not orginally modeled.
+#'   even if not originally modeled.
 #' @param conditions An optional \code{data.frame} containing variable values
 #'   to condition on. Each effect defined in \code{effects} will
 #'   be plotted separately for each row of \code{data}. 
@@ -836,7 +813,7 @@ stanplot <- function(object, ...) {
 #'   If \code{"fitted"}, plot marginal predictions of the regression curve. 
 #'   If \code{"predict"}, plot marginal predictions of the responses.
 #' @param spaghetti Logical; Indicates whether predictions should
-#'   be visualized via spagetti plots. Only applied for numeric
+#'   be visualized via spaghetti plots. Only applied for numeric
 #'   predictors. If \code{TRUE}, it is recommended 
 #'   to set argument \code{nsamples} to a relatively small value 
 #'   (e.g. \code{100}) in order to reduce computation time.
@@ -942,7 +919,7 @@ stanplot <- function(object, ...) {
 #'   This also has an implication for the \code{points} argument: 
 #'   In the created plots, only those points will be shown that correspond 
 #'   to the factor levels actually used in the conditioning, in order not 
-#'   to create the false impressivion of bad model fit, where it is just 
+#'   to create the false impression of bad model fit, where it is just 
 #'   due to conditioning on certain factor levels.
 #'   Since we condition on rather than actually marginalizing variables, 
 #'   the name  \code{marginal_effects} is possibly not ideally chosen in 
@@ -953,7 +930,7 @@ stanplot <- function(object, ...) {
 #'   zero. This allows, for instance, to make predictions of the grand mean 
 #'   when using sum coding. 
 #'   
-#'   To fully change colours of the created plots, 
+#'   To fully change colors of the created plots, 
 #'   one has to amend both \code{scale_colour} and \code{scale_fill}.
 #'   See \code{\link[ggplot2:scale_colour_grey]{scale_colour_grey}} or
 #'   \code{\link[ggplot2:scale_colour_gradient]{scale_colour_gradient}}
@@ -1077,13 +1054,13 @@ marginal_smooths <- function(x, ...) {
 #' If \code{summary = TRUE}, an N x E x K array,
 #' where N is the number of observations, K is the number
 #' of mixture components, and E is equal to \code{length(probs) + 2}.
-#' If \code{summary = FALSE}, an S x N x K arrary, where
+#' If \code{summary = FALSE}, an S x N x K array, where
 #' S is the number of posterior samples.
 #' 
 #' @details 
 #' The returned probabilities can be written as
 #' \eqn{P(Kn = k | Yn)}, that is the posterior probability 
-#' that observation n orginiates from component k. 
+#' that observation n originates from component k. 
 #' They are computed using Bayes' Theorem
 #' \deqn{P(Kn = k | Yn) = P(Yn | Kn = k) P(Kn = k) / P(Yn),}
 #' where \eqn{P(Yn | Kn = k)} is the (posterior) likelihood
