@@ -1,117 +1,158 @@
-# All functions in this file have the same arguments structure
+predict_internal <- function(draws, ...) {
+  UseMethod("predict_internal")
+}
+
+#' @export
+predict_internal.mvbrmsdraws <- function(draws, ...) {
+  if (length(draws$mvpars$rescor)) {
+    draws$mvpars$Mu <- get_Mu(draws)
+    draws$mvpars$Sigma <- get_Sigma(draws)
+    out <- predict_internal.brmsdraws(draws, ...)
+  } else {
+    out <- lapply(draws$resps, predict_internal, ...)
+    along <- ifelse(length(out) > 1L, 3, 2)
+    out <- do.call(abind, c(out, along = along))
+  }
+  out
+}
+
+#' @export
+predict_internal.brmsdraws <- function(draws, summary = TRUE, transform = NULL,
+                                       sort = FALSE, robust = FALSE, 
+                                       probs = c(0.025, 0.975), ...) {
+  for (dp in names(draws$dpars)) {
+    draws$dpars[[dp]] <- get_dpar(draws, dpar = dp)
+  }
+  predict_fun <- paste0("predict_", draws$f$fun)
+  predict_fun <- get(predict_fun, asNamespace("brms"))
+  N <- choose_N(draws)
+  out <- lapply(seq_len(N), predict_fun, draws = draws, ...)
+  if (grepl("_mv$", draws$f$fun)) {
+    out <- do.call(abind, c(out, along = 3))
+    out <- aperm(out, perm = c(1, 3, 2))
+    dimnames(out)[[3]] <- names(draws$resps)
+  } else {
+    out <- do.call(cbind, out) 
+  }
+  # percentage of invalid samples for truncated discrete models
+  # should always be zero for all other models
+  pct_invalid <- get_pct_invalid(out, lb = draws$data$lb, ub = draws$data$ub) 
+  if (pct_invalid >= 0.01) {
+    warning2(
+      round(pct_invalid * 100), "% of all predicted values ", 
+      "were invalid. Increasing argument 'ntrys' may help."
+    )
+  }
+  out <- reorder_obs(out, draws$old_order, sort = sort)
+  # transform predicted response samples before summarizing them 
+  if (!is.null(transform)) {
+    out <- do.call(transform, list(out))
+  }
+  if (summary) {
+    if (is_ordinal(draws$f) || is_categorical(draws$f)) {
+      # compute frequencies of categories 
+      out <- get_table(out, levels = seq_len(max(draws$data$ncat)))
+    } else {
+      out <- get_summary(out, probs = probs, robust = robust)
+    }
+  }
+  out
+}
+
+# All predict_<family> functions have the same arguments structure
 # Args:
 #  i: the column of draws to use i.e. the ith obervation 
 #     in the initial data.frame 
 #  draws: A named list returned by extract_draws containing 
 #         all required data and samples
-#  ntrys: number of trys in rejection sampling for truncated discrete models
 #  ...: ignored arguments
 # Returns:
 #   A vector of length draws$nsamples containing samples
 #   from the posterior predictive distribution
 predict_gaussian <- function(i, draws, ...) {
-  args <- list(mean = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sd = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "norm", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    mean = get_dpar(draws, "mu", i = i), 
+    sd = get_dpar(draws, "sigma", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "norm", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_student <- function(i, draws, ...) {
-  args <- list(df = get_dpar(draws$nu, i = i), 
-               mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sigma = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "student_t", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
-}
-
-predict_cauchy <- function(i, draws, ...) {
-  args <- list(df = 1, mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sigma = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "student_t", args = args,
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    df = get_dpar(draws, "nu", i = i), 
+    mu = get_dpar(draws, "mu", i = i), 
+    sigma = get_dpar(draws, "sigma", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "student_t", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_lognormal <- function(i, draws, ...) {
-  args <- list(meanlog = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sdlog = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "lnorm", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    meanlog = get_dpar(draws, "mu", i = i), 
+    sdlog = get_dpar(draws, "sigma", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "lnorm", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_skew_normal <- function(i, draws, ...) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, i = i)
-  alpha <- get_dpar(draws$alpha, i = i)
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
+  sigma <- get_dpar(draws, "sigma", i = i)
+  alpha <- get_dpar(draws, "alpha", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
   args <- nlist(mu, sigma, alpha)
-  rng_continuous(nrng = draws$nsamples, dist = "skew_normal", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  rng_continuous(
+    nrng = draws$nsamples, dist = "skew_normal", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_gaussian_mv <- function(i, draws, ...) {
-  # currently no truncation available
-  if (!is.null(draws$data$N_trait)) {
-    obs <- seq(i, draws$data$N, draws$data$N_trait)
-    mu <- ilink(get_eta(draws$mu, obs), draws$f$link)
-  } else {
-    mu <- ilink(get_eta(draws$mu, i), draws$f$link)[, 1, ]
+  Mu <- get_Mu(draws, i = i)
+  Sigma <- get_Sigma(draws, i = i)
+  rmn <- function(s) {
+    rmulti_normal(1, mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  .fun <- function(s) {
-    rmulti_normal(1, mu = mu[s, ], Sigma = draws$Sigma[s, , ])
-  }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(1:draws$nsamples, rmn))
 }
 
 predict_student_mv <- function(i, draws, ...) {
-  # currently no truncation available
-  if (!is.null(draws$data$N_trait)) {
-    obs <- seq(i, draws$data$N, draws$data$N_trait)
-    mu <- ilink(get_eta(draws$mu, obs), draws$f$link)
-  } else {
-    mu <- ilink(get_eta(draws$mu, i), draws$f$link)[, 1, ]
+  nu <- get_dpar(draws, "nu", i = i)
+  Mu <- get_Mu(draws, i = i)
+  Sigma <- get_Sigma(draws, i = i)
+  rmst <- function(s) {
+    rmulti_student_t(1, df = nu[s], mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  nu <- get_dpar(draws$nu, i = i)
-  .fun <- function(s) {
-    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], 
-                     Sigma = draws$Sigma[s, , ])
-  }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
-}
-
-predict_cauchy_mv <- function(i, draws, ...) {
-  # currently no truncation available
-  if (!is.null(draws$data$N_trait)) {
-    obs <- seq(i, draws$data$N, draws$data$N_trait)
-    mu <- ilink(get_eta(draws$mu, obs), draws$f$link)
-  } else {
-    mu <- ilink(get_eta(draws$mu, i), draws$f$link)[, 1, ]
-  }
-  .fun <- function(s) {
-    rmulti_student_t(1, df = 1, mu = mu[s, ],
-                     Sigma = draws$Sigma[s, , ])
-  }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(1:draws$nsamples, rmst))
 }
 
 predict_gaussian_cov <- function(i, draws, ...) {
   # currently, only ARMA1 processes are implemented
-  obs <- with(draws$data, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
-  mu <- ilink(get_eta(draws$mu, obs), draws$f$link)
-  args <- list(sigma = get_sigma(draws$sigma, data = draws$data, i = i), 
-               se2 = draws$data$se2[obs], nrows = length(obs))
-  if (!is.null(draws$ar) && is.null(draws$ma)) {
-    # AR1 process
-    args$ar <- draws$ar
+  obs <- with(draws$ac, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
+  mu <- as.matrix(get_dpar(draws, "mu", i = obs))
+  args <- list(
+    sigma = get_dpar(draws, "sigma", i = obs),
+    se = draws$data$se[obs], nrows = length(obs)
+  )
+  ar <- draws$ac$ar
+  ma <- draws$ac$ma
+  if (!is.null(ar) && is.null(ma)) {
+    args$ar <- ar
     Sigma <- do.call(get_cov_matrix_ar1, args)
-  } else if (is.null(draws$ar) && !is.null(draws$ma)) {
-    # MA1 process
-    args$ma <- draws$ma
+  } else if (is.null(ar) && !is.null(ma)) {
+    args$ma <- ma
     Sigma <- do.call(get_cov_matrix_ma1, args)
-  } else if (!is.null(draws$ar) && !is.null(draws$ma)) {
-    # ARMA1 process
-    args[c("ar", "ma")] <- draws[c("ar", "ma")]
+  } else if (!is.null(ar) && !is.null(ma)) {
+    args[c("ar", "ma")] <- draws$ac[c("ar", "ma")]
     Sigma <- do.call(get_cov_matrix_arma1, args)
   } else {
-    # identity matrix
     Sigma <- do.call(get_cov_matrix_ident, args)
   }
   .fun <- function(s) {
@@ -122,213 +163,233 @@ predict_gaussian_cov <- function(i, draws, ...) {
 
 predict_student_cov <- function(i, draws, ...) {
   # currently, only ARMA1 processes are implemented
-  obs <- with(draws$data, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
-  mu <- ilink(get_eta(draws$mu, obs), draws$f$link)
-  args <- list(sigma = get_sigma(draws$sigma, data = draws$data, i = i), 
-               se2 = draws$data$se2[obs], nrows = length(obs))
-  if (!is.null(draws$ar) && is.null(draws$ma)) {
-    # AR1 process
-    args$ar <- draws$ar
+  obs <- with(draws$ac, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
+  args <- list(
+    sigma = get_dpar(draws, "sigma", i = obs), 
+    se = draws$data$se[obs], nrows = length(obs)
+  )
+  ar <- draws$ac$ar
+  ma <- draws$ac$ma
+  if (!is.null(ar) && is.null(ma)) {
+    args$ar <- ar
     Sigma <- do.call(get_cov_matrix_ar1, args)
-  } else if (is.null(draws$ar) && !is.null(draws$ma)) {
-    # MA1 process
-    args$ma <- draws$ma
+  } else if (is.null(ar) && !is.null(ma)) {
+    args$ma <- ma
     Sigma <- do.call(get_cov_matrix_ma1, args)
-  } else if (!is.null(draws$ar) && !is.null(draws$ma)) {
-    # ARMA1 process
-    args[c("ar", "ma")] <- draws[c("ar", "ma")]
+  } else if (!is.null(ar) && !is.null(ma)) {
+    args[c("ar", "ma")] <- draws$ac[c("ar", "ma")]
     Sigma <- do.call(get_cov_matrix_arma1, args)
   } else {
-    # identity matrix
     Sigma <- do.call(get_cov_matrix_ident, args)
   }
-  nu <- get_dpar(draws$nu, i = i)
+  mu <- as.matrix(get_dpar(draws, "mu", i = obs))
+  nu <- as.matrix(get_dpar(draws, "nu", i = obs))
   .fun <- function(s) {
-    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], 
-                     Sigma = Sigma[s, , ])
+    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = Sigma[s, , ])
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
 
-predict_cauchy_cov <- function(i, draws, ...) {
-  draws$nu <- matrix(rep(1, draws$nsamples))
-  predict_student_cov(i = i, draws = draws, ...) 
-}
-
 predict_gaussian_lagsar <- function(i, draws, ...) {
   stopifnot(i == 1)
   .predict_gaussian_lagsar <- function(s) {
-    W_new <- with(draws, diag(data$N) - lagsar[s, ] * data$W)
+    W_new <- with(draws, diag(nobs) - ac$lagsar[s] * ac$W)
     mu <- as.numeric(solve(W_new) %*% mu[s, ])
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_normal(1, mu = mu, Sigma = Sigma)
   }
-  mu <- ilink(get_eta(draws$mu), draws$f$link)
-  sigma <- get_sigma(draws$sigma, data = draws$data)
+  mu <- get_dpar(draws, "mu")
+  sigma <- get_dpar(draws, "sigma")
   do.call(rbind, lapply(1:draws$nsamples, .predict_gaussian_lagsar))
 }
 
 predict_student_lagsar <- function(i, draws, ...) {
   stopifnot(i == 1)
   .predict_student_lagsar <- function(s) {
-    W_new <- with(draws, diag(data$N) - lagsar[s, ] * data$W)
+    W_new <- with(draws, diag(nobs) - ac$lagsar[s] * ac$W)
     mu <- as.numeric(solve(W_new) %*% mu[s, ])
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_student_t(1, df = nu[s], mu = mu, Sigma = Sigma)
   }
-  N <- draws$data$N
-  mu <- ilink(get_eta(draws$mu), draws$f$link)
-  sigma <- get_sigma(draws$sigma, data = draws$data)
-  nu <- get_dpar(draws$nu)
+  mu <- get_dpar(draws, "mu")
+  sigma <- get_dpar(draws, "sigma")
+  nu <- get_dpar(draws, "nu")
   do.call(rbind, lapply(1:draws$nsamples, .predict_student_lagsar))
 }
 
 predict_gaussian_errorsar <- function(i, draws, ...) {
   stopifnot(i == 1)
   .predict_gaussian_errorsar <- function(s) {
-    W_new <- with(draws, diag(data$N) - errorsar[s, ] * data$W)
+    W_new <- with(draws, diag(nobs) - ac$errorsar[s] * ac$W)
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_normal(1, mu = mu[s, ], Sigma = Sigma)
   }
-  mu <- ilink(get_eta(draws$mu), draws$f$link)
-  sigma <- get_sigma(draws$sigma, data = draws$data)
+  mu <- get_dpar(draws, "mu")
+  sigma <- get_dpar(draws, "sigma")
   do.call(rbind, lapply(1:draws$nsamples, .predict_gaussian_errorsar))
 }
 
 predict_student_errorsar <- function(i, draws, ...) {
   stopifnot(i == 1)
   .predict_student_errorsar <- function(s) {
-    W_new <- with(draws, diag(data$N) - errorsar[s, ] * data$W)
+    W_new <- with(draws, diag(nobs) - ac$errorsar[s] * ac$W)
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_student_t(1, df = nu[s], mu = mu[s, ], Sigma = Sigma)
   }
-  mu <- ilink(get_eta(draws$mu), draws$f$link)
-  sigma <- get_sigma(draws$sigma, data = draws$data)
-  nu <- get_dpar(draws$nu)
+  mu <- get_dpar(draws, "mu")
+  sigma <- get_dpar(draws, "sigma")
+  nu <- get_dpar(draws, "nu")
   do.call(rbind, lapply(1:draws$nsamples, .predict_student_errorsar))
 }
 
 predict_gaussian_fixed <- function(i, draws, ...) {
   stopifnot(i == 1)
-  mu <- ilink(get_eta(draws$mu, 1:nrow(draws$data$V)), draws$f$link)
+  mu <- as.matrix(get_dpar(draws, "mu"))
   .fun <- function(s) {
-    rmulti_normal(1, mu = mu[s, ], Sigma = draws$data$V)
+    rmulti_normal(1, mu = mu[s, ], Sigma = draws$ac$V)
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
 
 predict_student_fixed <- function(i, draws, ...) {
   stopifnot(i == 1)
-  mu <- ilink(get_eta(draws$mu, 1:nrow(draws$data$V)), draws$f$link)
-  nu <- get_dpar(draws$nu, 1:nrow(draws$data$V))
+  mu <- as.matrix(get_dpar(draws, "mu"))
+  nu <- as.matrix(get_dpar(draws, "nu"))
   .fun <- function(s) {
-    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$data$V)
+    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$ac$V)
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
 
-predict_cauchy_fixed <- function(i, draws, ...) {
-  stopifnot(i == 1)
-  draws$nu <- matrix(rep(1, draws$nsamples))
-  predict_student_fixed(i, draws = draws, ...)
-}
-
 predict_binomial <- function(i, draws, ntrys = 5, ...) {
-  trials <- draws$data$trials[i]
-  args <- list(size = trials, prob = ilink(get_eta(draws$mu, i), draws$f$link))
-  rng_discrete(nrng = draws$nsamples, dist = "binom", args = args, 
-               lb = draws$data$lb[i], ub = draws$data$ub[i], 
-               ntrys = ntrys)
+  args <- list(
+    size = draws$data$trials[i], 
+    prob = get_dpar(draws, "mu", i = i)
+  )
+  rng_discrete(
+    nrng = draws$nsamples, dist = "binom", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i], 
+    ntrys = ntrys
+  )
 }
 
 predict_bernoulli <- function(i, draws, ...) {
   # truncation not useful
-  eta <- get_eta(draws$mu, i)
-  rbinom(length(eta), size = 1, prob = ilink(eta, draws$f$link))
+  mu <- get_dpar(draws, "mu", i = i)
+  rbinom(length(mu), size = 1, prob = mu)
 }
 
 predict_poisson <- function(i, draws, ntrys = 5, ...) {
-  args <- list(lambda = ilink(get_eta(draws$mu, i), draws$f$link))
-  rng_discrete(nrng = draws$nsamples, dist = "pois", args = args, 
-               lb = draws$data$lb[i], ub = draws$data$ub[i],
-               ntrys = ntrys)
+  args <- list(lambda = get_dpar(draws, "mu", i = i))
+  rng_discrete(
+    nrng = draws$nsamples, dist = "pois", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i],
+    ntrys = ntrys
+  )
 }
 
 predict_negbinomial <- function(i, draws, ntrys = 5, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               size = get_shape(draws$shape, data = draws$data, i = i))
-  rng_discrete(nrng = draws$nsamples, dist = "nbinom", args = args, 
-               lb = draws$data$lb[i], ub = draws$data$ub[i],
-               ntrys = ntrys)
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    size = get_dpar(draws, "shape", i = i)
+  )
+  rng_discrete(
+    nrng = draws$nsamples, dist = "nbinom", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i],
+    ntrys = ntrys
+  )
 }
 
 predict_geometric <- function(i, draws, ntrys = 5, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), size = 1)
-  rng_discrete(nrng = draws$nsamples, dist = "nbinom", args = args, 
-               lb = draws$data$lb[i], ub = draws$data$ub[i], 
-               ntrys = ntrys)
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    size = 1
+  )
+  rng_discrete(
+    nrng = draws$nsamples, dist = "nbinom", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i], 
+    ntrys = ntrys
+  )
 }
 
 predict_exponential <- function(i, draws, ...) {
-  args <- list(rate = 1 / ilink(get_eta(draws$mu, i), draws$f$link))
-  rng_continuous(nrng = draws$nsamples, dist = "exp", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(rate = 1 / get_dpar(draws, "mu", i = i))
+  rng_continuous(
+    nrng = draws$nsamples, dist = "exp", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_gamma <- function(i, draws, ...) {
-  shape <- get_shape(draws$shape, data = draws$data, i = i)
-  args <- list(shape = shape, 
-               scale = ilink(get_eta(draws$mu, i), draws$f$link) / shape)
-  rng_continuous(nrng = draws$nsamples, dist = "gamma", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  shape <- get_dpar(draws, "shape", i = i)
+  args <- nlist(shape, scale = get_dpar(draws, "mu", i = i) / shape)
+  rng_continuous(
+    nrng = draws$nsamples, dist = "gamma", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_weibull <- function(i, draws, ...) {
-  shape <- get_shape(draws$shape, data = draws$data, i = i)
-  scale <- ilink(get_eta(draws$mu, i) / shape, draws$f$link)
+  shape <- get_dpar(draws, "shape", i = i)
+  mu <- get_dpar(draws, "mu", i = i) 
+  scale <- ilink(mu / shape, draws$f$link)
   args <- list(shape = shape, scale = scale)
-  rng_continuous(nrng = draws$nsamples, dist = "weibull", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  rng_continuous(
+    nrng = draws$nsamples, dist = "weibull", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_frechet <- function(i, draws, ...) {
-  nu <- get_dpar(draws$nu, i = i)
-  scale <- ilink(get_eta(draws$mu, i), draws$f$link) / gamma(1 - 1 / nu)
+  nu <- get_dpar(draws, "nu", i = i)
+  scale <- get_dpar(draws, "mu", i = i) / gamma(1 - 1 / nu)
   args <- list(scale = scale, shape = nu)
-  rng_continuous(nrng = draws$nsamples, dist = "frechet", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  rng_continuous(
+    nrng = draws$nsamples, dist = "frechet", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_gen_extreme_value <- function(i, draws, ...) {
-  sigma <- get_sigma(draws$sigma, data = draws$data, i = i)
-  xi <- get_dpar(draws$xi, i = i)
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
+  sigma <- get_dpar(draws, "sigma", i = i)
+  xi <- get_dpar(draws, "xi", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
   args <- nlist(mu, sigma, xi)
-  rng_continuous(nrng = draws$nsamples, dist = "gen_extreme_value", 
-                 args = args, lb = draws$data$lb[i], ub = draws$data$ub[i])
+  rng_continuous(
+    nrng = draws$nsamples, dist = "gen_extreme_value", 
+    args = args, lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_inverse.gaussian <- function(i, draws, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               shape = get_shape(draws$shape, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "inv_gaussian", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    shape = get_dpar(draws, "shape", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "inv_gaussian", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_exgaussian <- function(i, draws, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sigma = get_sigma(draws$sigma, data = draws$data, i = i),
-               beta = get_dpar(draws$beta, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "exgaussian", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    sigma = get_dpar(draws, "sigma", i = i),
+    beta = get_dpar(draws, "beta", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "exgaussian", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_wiener <- function(i, draws, negative_rt = FALSE, ...) {
   args <- list(
-    delta = ilink(get_eta(draws$mu, i), draws$f$link), 
-    alpha = get_dpar(draws$bs, i = i),
-    tau = get_dpar(draws$ndt, i = i),
-    beta = get_dpar(draws$bias, i = i),
+    delta = get_dpar(draws, "mu", i = i), 
+    alpha = get_dpar(draws, "bs", i = i),
+    tau = get_dpar(draws, "ndt", i = i),
+    beta = get_dpar(draws, "bias", i = i),
     types = if (negative_rt) c("q", "resp") else "q"
   )
   out <- rng_continuous(
@@ -343,32 +404,42 @@ predict_wiener <- function(i, draws, negative_rt = FALSE, ...) {
 }
 
 predict_beta <- function(i, draws, ...) {
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
-  phi <- get_dpar(draws$phi, i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  phi <- get_dpar(draws, "phi", i = i)
   args <- list(shape1 = mu * phi, shape2 = (1 - mu) * phi)
-  rng_continuous(nrng = draws$nsamples, dist = "beta", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  rng_continuous(
+    nrng = draws$nsamples, dist = "beta", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_von_mises <- function(i, draws, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               kappa = get_dpar(draws$kappa, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "von_mises", args = args,
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    kappa = get_dpar(draws, "kappa", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "von_mises", args = args,
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_asym_laplace <- function(i, draws, ...) {
-  args <- list(mu = ilink(get_eta(draws$mu, i), draws$f$link), 
-               sigma = get_sigma(draws$sigma, data = draws$data, i = i),
-               quantile = get_dpar(draws$quantile, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "asym_laplace", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+  args <- list(
+    mu = get_dpar(draws, "mu", i = i), 
+    sigma = get_dpar(draws, "sigma", i = i),
+    quantile = get_dpar(draws, "quantile", i = i)
+  )
+  rng_continuous(
+    nrng = draws$nsamples, dist = "asym_laplace", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
 }
 
 predict_hurdle_poisson <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_zi_hu(draws, i, par = "hu")
-  lambda <- ilink(get_eta(draws$mu, i), draws$f$link)
+  theta <- get_dpar(draws, "hu", i = i) 
+  lambda <- get_dpar(draws, "mu", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
   hu <- runif(ndraws, 0, 1)
@@ -380,23 +451,23 @@ predict_hurdle_poisson <- function(i, draws, ...) {
 
 predict_hurdle_negbinomial <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_zi_hu(draws, i, par = "hu")
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
+  theta <- get_dpar(draws, "hu", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
   hu <- runif(ndraws, 0, 1)
-  # sample from an approximative(!) truncated negbinomial distribution
+  # sample from an approximate(!) truncated negbinomial distribution
   # by adjusting mu and adding 1
   t = -log(1 - runif(ndraws) * (1 - exp(-mu)))
-  shape <- get_shape(draws$shape, data = draws$data, i = i)
+  shape <- get_dpar(draws, "shape", i = i)
   ifelse(hu < theta, 0, rnbinom(ndraws, mu = mu - t, size = shape) + 1)
 }
 
 predict_hurdle_gamma <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_zi_hu(draws, i, par = "hu")
-  shape <- get_shape(draws$shape, data = draws$data, i = i)
-  scale <- ilink(get_eta(draws$mu, i), draws$f$link) / shape
+  theta <- get_dpar(draws, "hu", i = i)
+  shape <- get_dpar(draws, "shape", i = i)
+  scale <- get_dpar(draws, "mu", i = i) / shape
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
   hu <- runif(ndraws, 0, 1)
@@ -405,9 +476,9 @@ predict_hurdle_gamma <- function(i, draws, ...) {
 
 predict_hurdle_lognormal <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_zi_hu(draws, i, par = "hu")
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
-  sigma <- get_sigma(draws$sigma, data = draws$data, i = i)
+  theta <- get_dpar(draws, "hu", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  sigma <- get_dpar(draws, "sigma", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
   hu <- runif(ndraws, 0, 1)
@@ -416,20 +487,22 @@ predict_hurdle_lognormal <- function(i, draws, ...) {
 
 predict_zero_inflated_beta <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_zi_hu(draws, i, par = "zi")
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
-  phi <- get_dpar(draws$phi, i = i)
+  theta <- get_dpar(draws, "zi", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  phi <- get_dpar(draws, "phi", i = i)
   # compare with theta to incorporate the hurdle process
   hu <- runif(draws$nsamples, 0, 1)
-  ifelse(hu < theta, 0, rbeta(draws$nsamples, shape1 = mu * phi, 
-                              shape2 = (1 - mu) * phi))
+  ifelse(
+    hu < theta, 0, 
+    rbeta(draws$nsamples, shape1 = mu * phi, shape2 = (1 - mu) * phi)
+  )
 }
 
 predict_zero_one_inflated_beta <- function(i, draws, ...) {
-  zoi <- get_dpar(draws$zoi, i)
-  coi <- get_dpar(draws$coi, i)
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
-  phi <- get_dpar(draws$phi, i = i)
+  zoi <- get_dpar(draws, "zoi", i)
+  coi <- get_dpar(draws, "coi", i)
+  mu <- get_dpar(draws, "mu", i = i)
+  phi <- get_dpar(draws, "phi", i = i)
   hu <- runif(draws$nsamples, 0, 1)
   one_or_zero <- runif(draws$nsamples, 0, 1)
   ifelse(hu < zoi, 
@@ -440,8 +513,8 @@ predict_zero_one_inflated_beta <- function(i, draws, ...) {
 
 predict_zero_inflated_poisson <- function(i, draws, ...) {
   # theta is the bernoulli zero-inflation parameter
-  theta <- get_zi_hu(draws, i, par = "zi")
-  lambda <- ilink(get_eta(draws$mu, i), draws$f$link)
+  theta <- get_dpar(draws, "zi", i = i)
+  lambda <- get_dpar(draws, "mu", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the zero-inflation process
   zi <- runif(ndraws, 0, 1)
@@ -450,9 +523,9 @@ predict_zero_inflated_poisson <- function(i, draws, ...) {
 
 predict_zero_inflated_negbinomial <- function(i, draws, ...) {
   # theta is the bernoulli zero-inflation parameter
-  theta <- get_zi_hu(draws, i, par = "zi")
-  mu <- ilink(get_eta(draws$mu, i), draws$f$link)
-  shape <- get_shape(draws$shape, data = draws$data, i = i)
+  theta <- get_dpar(draws, "zi", i = i)
+  mu <- get_dpar(draws, "mu", i = i)
+  shape <- get_dpar(draws, "shape", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the zero-inflation process
   zi <- runif(ndraws, 0, 1)
@@ -461,9 +534,9 @@ predict_zero_inflated_negbinomial <- function(i, draws, ...) {
 
 predict_zero_inflated_binomial <- function(i, draws, ...) {
   # theta is the bernoulii zero-inflation parameter
-  theta <- get_zi_hu(draws, i, par = "zi")
+  theta <- get_dpar(draws, "zi", i = i)
   trials <- draws$data$trials[i]
-  prob <- ilink(get_eta(draws$mu, i), draws$f$link)
+  prob <- get_dpar(draws, "mu", i = i)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the zero-inflation process
   zi <- runif(ndraws, 0, 1)
@@ -471,9 +544,8 @@ predict_zero_inflated_binomial <- function(i, draws, ...) {
 }
 
 predict_categorical <- function(i, draws, ...) {
-  ncat <- draws$data$ncat
-  p <- pcategorical(1:ncat, eta = get_eta(draws$mu, i)[, 1, ], 
-                    ncat = ncat, link = draws$f$link)
+  eta <- sapply(names(draws$dpars), get_dpar, draws = draws, i = i)
+  p <- pcategorical(seq_len(draws$data$ncat), eta = eta)
   first_greater(p, target = runif(draws$nsamples, min = 0, max = 1))
 }
 
@@ -495,10 +567,12 @@ predict_acat <- function(i, draws, ...) {
 
 predict_ordinal <- function(i, draws, family, ...) {
   ncat <- draws$data$ncat
-  disc <- get_disc(draws, i, ncat)
-  eta <- (disc * get_eta(draws$mu, i))[, 1, ]
-  p <- pordinal(1:ncat, eta = eta, ncat = ncat, 
-                family = family, link = draws$f$link)
+  disc <- get_dpar(draws, "disc", i = i)
+  eta <- (disc * get_dpar(draws, "mu", i = i))
+  p <- pordinal(
+    seq_len(ncat), eta = eta, ncat = ncat, 
+    family = family, link = draws$f$link
+  )
   first_greater(p, target = runif(draws$nsamples, min = 0, max = 1))
 }
 
@@ -512,15 +586,7 @@ predict_mixture <- function(i, draws, ...) {
     if (length(sample_ids)) {
       predict_fun <- paste0("predict_", families[j])
       predict_fun <- get(predict_fun, asNamespace("brms"))
-      dpars <- valid_dpars(families[j])
-      tmp_draws <- list(
-        f = draws$f$mix[[j]],
-        nsamples = length(sample_ids),
-        data = draws[["data"]]
-      )
-      for (ap in dpars) {
-        tmp_draws[[ap]] <- p(draws[[paste0(ap, j)]], sample_ids, row = TRUE)
-      }
+      tmp_draws <- pseudo_draws_for_mixture(draws, j, sample_ids)
       out[sample_ids] <- predict_fun(i, tmp_draws, ...)
     }
   }
@@ -586,7 +652,6 @@ rng_discrete <- function(nrng, dist, args, lb = NULL, ub = NULL, ntrys = 5) {
 
 rng_mix <- function(theta) {
   # sample the ID of the mixture component
-  #stopifnot(all(dim(theta), c(nrng, length(families))))
   stopifnot(is.matrix(theta))
   mix_comp <- seq_len(ncol(theta))
   ulapply(seq_len(nrow(theta)), function(s)
@@ -629,4 +694,4 @@ get_pct_invalid <- function(x, lb = NULL, ub = NULL) {
   } else {
     0
   }
-}
\ No newline at end of file
+}
